' ============================================================================
' qstat_NeM.vbs		v2.0
' ============================================================================


' Game codes

' qs             query Quake server
' h2s            query Hexen II server
' qws            query QuakeWorld server
' hws            query HexenWorld server
' q2s            query Quake II server
' q3s            query Quake III: Arena server
' rws            query Return to Castle Wolfenstein server
' efs            query Star Trek: Elite Force server
' ut2s           query Unreal Tournament 2003 server
' uns            query Unreal server
' hls            query Half-Life server
' sns            query Sin server
' sgs            query Shogo: Mobile Armor Division server
' tbs            query Tribes server
' bfs            query BFRIS server
' kps            query Kingpin server
' hrs            query Heretic II server
' sfs            query Soldier of Fortune server
' gps            query Gamespy Protocol server
' t2s            query Tribes 2 server
' d3g            query Descent3 Gamespy Protocol server
' d3s            query Descent3 server
' d3p            query Descent3 PXO protocol server
' grs            query Ghost Recon server
' qwm            query QuakeWorld Master server
' q2m            query Quake II Master server
' q3m            query Quake III Master server
' rwm            query Return to Castle Wolfenstein Master server
' efm            query Star Trek: Elite Force server
' hlm            query Half-Life Master server
' tbm            query Tribes Master server
' gsm            query Gamespy Master server
' t2m            query Tribes 2 Master server
' d3m            query Descent3 Master (PXO) server
' sof2s          query Soldier of Fortune 2 server
' sof2m          query SOF2 Master server
' sof2m1.0       query SOF2 Master (1.0) server
' crs            query Command and Conquer: Renegade server
' mas            query Medal of Honor: Allied Assault server
' maqs           query Medal of Honor: Allied Assault (Q) server
' hlqs           query Half-Life server
' bds            query Battlefield 1942 server

'Changes in v2.0:
'================

' - rewritten from scratch
' - uses qstat XML dump
' - loads more options :)
' - should be more robust

' ==================================================================
' USER SETTINGS
' ==================================================================

' Path to your Samurize directory
SAMURIZE_PATH = "C:\progra~1\samurize"

UPDATE_INTERVAL = 0	' num. of minutes before server stats update
			' 0 = Script Interval rate in Samurize

UPDATE_TIME_FORMAT = 3	' how you want the last updated time format
			' to appear:
				' 0 = 8/04/2003 3:04:02 p.m.
				' 1 = Tuesday, 8 April 2003
				' 2 = 8/04/2003
				' 3 = 3:20:08 p.m.
				' 4 = 15:20

TIMEOUT = 30		' timeout interval (seconds)

ERROR_MSG = "ERROR"	' Message displayed when there is an error

' ==================================================================



SERVER_LIST = SAMURIZE_PATH & "\qstat\serverlist.txt"
SERVER_STATUS = SAMURIZE_PATH & "\qstat\serverstatus.txt"
SERVER_ATTLIST = SAMURIZE_PATH & "\qstat\server."
SERVER_PLAYERLIST = SAMURIZE_PATH & "\qstat\players."
QSTAT_PATH = SAMURIZE_PATH & "\qstat\qstat.exe"
Set MyShell = createobject("WScript.Shell")


' ***************************************************
' *           Samurize functions follow             *
' ***************************************************

Function updateServerStatus
    Dim fs,f
    diff = TimeDiff(SERVER_STATUS)
    if (diff > UPDATE_INTERVAL) or (diff = -1) then
	'update server stats
	oExec = MyShell.Run(QSTAT_PATH & " -f " & SERVER_LIST & " -sort l -xml -P -R -of " & SERVER_STATUS & " -timeout " & TIMEOUT,0,True)
    End If
	
    ' report last update time
    Set fs=CreateObject("Scripting.FileSystemObject")
    If fs.fileexists(SERVER_STATUS) Then
	Set f = fs.GetFile(SERVER_STATUS)
	updateServerStatus = f.DateLastModified
	updateServerStatus = CStr(FormatDateTime(updateServerStatus, UPDATE_TIME_FORMAT))
	ComputeServerStuff()
    Else
	updateServerStatus = "ERROR"
    End If
    
End Function



' ***************************************************
' *         Auxiliary functions follow              *
' ***************************************************

Private Function ComputeServerStuff
    Dim objXML, ServerList, f, fs
    
    Set objXML = CreateObject("Microsoft.XMLDOM")
    objXML.async = False
    objXML.Load(SERVER_STATUS)
    
    If objXML.parseError.errorCode <> 0 Then
	ComputeServerStuff = "parse error"
	exit function
    End If

    ' Get list of all Servers
    Set ServerList = objXML.getElementsByTagName("server")
    
    For i=1 to serverlist.length
	ComputeServerStuff = ComputeServerStuff & i
	set fs=CreateObject("Scripting.FileSystemObject")
	set f=fs.CreateTextFile(SERVER_ATTLIST&i,true)
	
	ServerStats = ServerType(i-1) & vbcrlf & Address(i-1) & vbcrlf & Status(i-1) & vbcrlf & HostName(i-1) & vbcrlf & ServerName(i-1) & _
			vbcrlf & GameType(i-1) & vbcrlf & MapName(i-1) & vbcrlf & NumberOfPlayers(i-1) & vbcrlf & MaxPlayers(i-1) & vbcrlf & PercentPlayers(i-1) & vbcrlf & _
			Ping(i-1) & vbcrlf
	
	f.write(ServerStats)
	f.close
	
	Set f = fs.CreateTextFile(SERVER_PLAYERLIST&i,true)
	f.write(GeneratePlayerList(i-1))
	f.close
	
	set f=nothing
	set fs=nothing
    Next
End Function


Private Function ServerType(serverNum)
    ServerType = GetServerInfo(ServerNum, "","","type")
End Function

Private Function Address(serverNum)
    Address = GetServerInfo(ServerNum, "","","address")
End Function

Private Function Status(serverNum)
    Status = GetServerInfo(ServerNum, "","","status")
End Function


Private Function HostName(serverNum)
    HostName = GetServerInfo(ServerNum, "hostname", "", "")
End Function


Private Function ServerName(serverNum)
    ServerName = GetServerInfo(ServerNum, "name", "", "")
End Function


Private Function GameType(serverNum)
    GameType = GetServerInfo(ServerNum, "gametype", "", "")
End Function


Private Function MapName(serverNum)
    MapName = GetServerInfo(ServerNum, "map","","")
End Function

Private Function numberOfPlayers(serverNum)
    numberofPlayers = GetServerInfo(ServerNum, "numplayers","","")
End Function

Private Function MaxPlayers(serverNum)
    MaxPlayers = GetServerInfo(ServerNum, "maxplayers","","")
End Function

Private Function ping(serverNum)
    ping = GetServerInfo(ServerNum, "ping", "", "")
End Function

Private Function PercentPlayers(serverNum)
    Num = numberOfPlayers(serverNum)
    Total = MaxPlayers(serverNum)
    
    If Num=ERROR_MSG then Num=0
    If Total=ERROR_MSG then Total=1
    
    PercentPlayers = Num & " " & Total
    PercentPlayers=CDbl((Num)/(Total)*100)
    
End Function


Private Function TimeDiff(file)
    dim fs,f,diff
    set fs=CreateObject("Scripting.FileSystemObject")
    if fs.fileexists(file) then
	set f=fs.GetFile(file)
        diff = datediff( "n", f.DateLastModified, now())
    else
        diff = -1
    end if
    set f=nothing
    set fs=nothing
    TimeDiff = diff
End Function


Private Function GeneratePlayerList(Servernum)

    Dim objXML, ServerList, Server, PlayerList, Player
    Dim PlayerAtt, PlayerAttList
    
    Set objXML = CreateObject("Microsoft.XMLDOM")
    objXML.async = False
    objXML.Load(SERVER_STATUS)
    
    If objXML.parseError.errorCode <> 0 Then
	GetServerInfo = "Parse error"
	exit function
    End If

    ' Get list of all Servers
    Set ServerList = objXML.getElementsByTagName("server")
    
    ' Check the requested server exists
    if ServerList.Length < (servernum + 1) then
	GetServerInfo = ERROR_MSG
	Exit Function
    end if
    
    Set Server = ServerList.item(servernum)


    Set PlayerList = Server.selectNodes("./players/player")

    For Each Player in PlayerList
	Set PlayerAttList = Player.ChildNodes
	For Each PlayerAtt in PlayerAttList
	    PlayerListTxt = PlayerListTxt & PlayerAtt.text & vbTab
	Next
	PlayerListTxt = PlayerListTxt & vbCrLf
    Next

    GeneratePlayerList = PlayerListTxt
    
End Function



' =======================================================================
' GetServerInfo
'
' ServerNum 	- the number of the server to query
' NodePath	- the path to the node from <server>
' NodeNum	- the node number to retrieve (if more than one exists)
' Attrib	- the name of the attribute to retrieve (if any)
' =======================================================================

Private Function GetServerInfo(Servernum, NodePath, NodeNum, Attrib)

    Dim objXML, ServerList, Server, ServerAttrib
    Dim ServerNodeList, ServerNode, AttList, AttNode
    
    Set objXML = CreateObject("Microsoft.XMLDOM")
    objXML.async = False
    objXML.Load(SERVER_STATUS)
    
    If objXML.parseError.errorCode <> 0 Then
	GetServerInfo = "Parse error"
	exit function
    End If

    ' Get list of all Servers
    Set ServerList = objXML.getElementsByTagName("server")
    
    ' Check the requested server exists
    if ServerList.Length < (servernum + 1) then
	GetServerInfo = ERROR_MSG
	Exit Function
    end if
    
    Set Server = ServerList.item(servernum)

    ' append slash if required
    If NodePath <> "" then NodePath = "/" & NodePath
    
    ' Get list of nodes in the supplied path
    Set ServerNodeList = Server.selectNodes("." & NodePath)
    
    ' If no NodeNum supplied, assume 1st node
    if nodenum = "" then nodenum = 0
    
    Set ServerNode = ServerNodeList.item(nodenum)

    ' Check node exists
    If ServerNode is Nothing then
	GetServerInfo = ERROR_MSG
	Exit Function
    End If
    
    If attrib = "" then
	' Return Node text
	GetServerInfo = ServerNode.text
	Exit Function
    end if 
    
    ' Get value of attribute
    Set AttList = ServerNode.attributes
    Set AttNode = AttList.GetNamedItem(attrib)
    
    if AttNode is Nothing then
	' Attribute not found
	GetServerInfo = ERROR_MSG
	Exit Function
    End If
    
    ' Return Attribute text
    GetServerInfo = AttNode.text
    
End Function


